@@ -27,7 +27,7 @@ use sql_gen::{
 mod operators;
 mod rewriter;
 
-/// Declare a function as `#[pg_guard]` to indicate that it is called from a Postgres `extern "C"`
+/// Declare a function as `#[pg_guard]` to indicate that it is called from a Postgres `extern "C-unwind"`
 /// function so that Rust `panic!()`s (and Postgres `elog(ERROR)`s) will be properly handled by `pgrx`
 #[proc_macro_attribute]
 pub fn pg_guard(_attr: TokenStream, item: TokenStream) -> TokenStream {
@@ -35,7 +35,7 @@ pub fn pg_guard(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let ast = parse_macro_input!(item as syn::Item);
 
     let res = match ast {
-        // this is for processing the members of extern "C" { } blocks
+        // this is for processing the members of extern "C-unwind" { } blocks
         // functions inside the block get wrapped as public, top-level unsafe functions that are not "extern"
         Item::ForeignMod(block) => Ok(rewriter::extern_block(block)),
 
@@ -43,7 +43,7 @@ pub fn pg_guard(_attr: TokenStream, item: TokenStream) -> TokenStream {
         Item::Fn(func) => rewriter::item_fn_without_rewrite(func),
         unknown => Err(syn::Error::new(
             unknown.span(),
-            "#[pg_guard] can only be applied to extern \"C\" blocks and top-level functions",
+            "#[pg_guard] can only be applied to extern \"C-unwind\" blocks and top-level functions",
         )),
     };
     res.unwrap_or_else(|e| e.into_compile_error()).into()
@@ -208,7 +208,7 @@ pub fn pg_cast(attr: TokenStream, item: TokenStream) -> TokenStream {
             }
         }
 
-        let pg_extern = PgExtern::new(pg_extern_attrs.into(), item.clone().into())?.0;
+        let pg_extern = PgExtern::new(pg_extern_attrs, item.clone().into())?.0;
         Ok(CodeEnrichment(pg_extern.as_cast(cast.unwrap_or_default())).to_token_stream().into())
     }
 
@@ -724,7 +724,7 @@ fn impl_postgres_enum(ast: DeriveInput) -> syn::Result<proc_macro2::TokenStream>
             type As<'dat> = #enum_ident #generics where Self: 'dat;
             #[inline]
             unsafe fn unbox<'dat>(d: ::pgrx::datum::Datum<'dat>) -> Self::As<'dat> where Self: 'dat {
-                Self::from_datum(::core::mem::transmute(d), false).unwrap()
+                <Self as ::pgrx::datum::FromDatum>::from_datum(::core::mem::transmute(d), false).unwrap()
             }
         }
 
@@ -908,7 +908,7 @@ fn impl_postgres_type(ast: DeriveInput) -> syn::Result<proc_macro2::TokenStream>
                             memory_context.switch_to(|_| {
                                 // this gets the varlena Datum copied into this memory context
                                 let varlena = ::pgrx::pg_sys::pg_detoast_datum_copy(datum.cast_mut_ptr());
-                                Self::from_datum(varlena.into(), is_null)
+                                <Self as ::pgrx::datum::FromDatum>::from_datum(varlena.into(), is_null)
                             })
                         }
                     }
