@@ -12,6 +12,7 @@ use eyre::{eyre, WrapErr};
 use owo_colors::OwoColorize;
 use serde::{Deserialize, Serialize};
 use std::collections::{BTreeMap, HashMap};
+use std::env::consts::EXE_SUFFIX;
 use std::ffi::OsString;
 use std::fmt::{self, Debug, Display, Formatter};
 use std::io::ErrorKind;
@@ -29,11 +30,7 @@ pub static BASE_POSTGRES_TESTING_PORT_NO: u16 = 32200;
 /// The flags to specify to get a "C.UTF-8" locale on this system, or "C" locale on systems without
 /// a "C.UTF-8" locale equivalent.
 pub fn get_c_locale_flags() -> &'static [&'static str] {
-    #[cfg(target_os = "macos")]
-    {
-        &["--locale=C", "--lc-ctype=UTF-8"]
-    }
-    #[cfg(not(target_os = "macos"))]
+    #[cfg(all(target_family = "unix", not(target_os = "macos")))]
     {
         match Command::new("locale").arg("-a").output() {
             Ok(cmd)
@@ -47,6 +44,14 @@ pub fn get_c_locale_flags() -> &'static [&'static str] {
             _ => &["--locale=C"],
         }
     }
+    #[cfg(target_os = "macos")]
+    {
+        &["--locale=C", "--lc-ctype=UTF-8"]
+    }
+    #[cfg(target_os = "windows")]
+    {
+        &["--locale=C"]
+    }
 }
 
 // These methods were originally in `pgrx-utils`, but in an effort to consolidate
@@ -330,35 +335,55 @@ impl PgConfig {
 
     pub fn postmaster_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
-        path.push("postgres");
-
+        path.push(format!("postgres{EXE_SUFFIX}"));
         Ok(path)
     }
 
     pub fn initdb_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
-        path.push("initdb");
+        path.push(format!("initdb{EXE_SUFFIX}"));
         Ok(path)
     }
 
     pub fn createdb_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
-        path.push("createdb");
+        path.push(format!("createdb{EXE_SUFFIX}"));
         Ok(path)
     }
 
     pub fn dropdb_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
-        path.push("dropdb");
+        path.push(format!("dropdb{EXE_SUFFIX}"));
+        Ok(path)
+    }
+
+    pub fn pg_ctl_path(&self) -> eyre::Result<PathBuf> {
+        let mut path = self.bin_dir()?;
+        path.push(format!("pg_ctl{EXE_SUFFIX}"));
         Ok(path)
     }
 
     pub fn psql_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
-        path.push("psql");
+        path.push(format!("psql{EXE_SUFFIX}"));
         Ok(path)
     }
 
+    pub fn pg_regress_path(&self) -> eyre::Result<PathBuf> {
+        let mut pgxs_path = self.pgxs_path()?;
+        pgxs_path.pop(); // pop the `pgxs.mk` file at the end
+        pgxs_path.pop(); // pop the `makefiles` directory in which it lives
+        let mut pgregress_path = pgxs_path;
+        pgregress_path.push("test");
+        pgregress_path.push("regress");
+        pgregress_path.push("pg_regress");
+        Ok(pgregress_path)
+    }
+
+    pub fn pgxs_path(&self) -> eyre::Result<PathBuf> {
+        self.run("--pgxs").map(PathBuf::from)
+    }
+
     pub fn data_dir(&self) -> eyre::Result<PathBuf> {
         let mut path = Pgrx::home()?;
         path.push(format!("data-{}", self.major_version()?));
@@ -385,10 +410,24 @@ impl PgConfig {
             .collect())
     }
 
+    pub fn pkgincludedir(&self) -> eyre::Result<PathBuf> {
+        Ok(self.run("--pkgincludedir")?.into())
+    }
+
     pub fn includedir_server(&self) -> eyre::Result<PathBuf> {
         Ok(self.run("--includedir-server")?.into())
     }
 
+    pub fn includedir_server_port_win32(&self) -> eyre::Result<PathBuf> {
+        let includedir_server = self.includedir_server()?;
+        Ok(includedir_server.join("port").join("win32"))
+    }
+
+    pub fn includedir_server_port_win32_msvc(&self) -> eyre::Result<PathBuf> {
+        let includedir_server = self.includedir_server()?;
+        Ok(includedir_server.join("port").join("win32_msvc"))
+    }
+
     pub fn pkglibdir(&self) -> eyre::Result<PathBuf> {
         Ok(self.run("--pkglibdir")?.into())
     }
@@ -663,7 +702,6 @@ impl Pgrx {
 #[allow(non_snake_case)]
 pub fn SUPPORTED_VERSIONS() -> Vec<PgVersion> {
     vec![
-        PgVersion::new(12, PgMinorVersion::Latest, None),
         PgVersion::new(13, PgMinorVersion::Latest, None),
         PgVersion::new(14, PgMinorVersion::Latest, None),
         PgVersion::new(15, PgMinorVersion::Latest, None),
@@ -687,7 +725,7 @@ pub fn createdb(
         return Ok(false);
     }
 
-    println!("{} database {}", "     Creating".bold().green(), dbname);
+    println!("{} database {}", "    Creating".bold().green(), dbname.bold().cyan());
     let createdb_path = pg_config.createdb_path()?;
     let mut command = if let Some(runas) = runas {
         let mut cmd = Command::new("sudo");
@@ -737,6 +775,66 @@ pub fn createdb(
     Ok(true)
 }
 
+pub fn dropdb(
+    pg_config: &PgConfig,
+    dbname: &str,
+    is_test: bool,
+    runas: Option<String>,
+) -> eyre::Result<bool> {
+    if !does_db_exist(pg_config, dbname)? {
+        return Ok(false);
+    }
+
+    println!("{} database {}", "    Dropping".bold().green(), dbname.bold().cyan());
+    let createdb_path = pg_config.dropdb_path()?;
+    let mut command = if let Some(runas) = runas {
+        let mut cmd = Command::new("sudo");
+        cmd.arg("-u").arg(runas).arg(createdb_path);
+        cmd
+    } else {
+        Command::new(createdb_path)
+    };
+    command
+        .env_remove("PGDATABASE")
+        .env_remove("PGHOST")
+        .env_remove("PGPORT")
+        .env_remove("PGUSER")
+        .arg("-h")
+        .arg(pg_config.host())
+        .arg("-p")
+        .arg(if is_test {
+            pg_config.test_port()?.to_string()
+        } else {
+            pg_config.port()?.to_string()
+        })
+        .arg(dbname)
+        .stdout(Stdio::piped())
+        .stderr(Stdio::piped());
+
+    let command_str = format!("{command:?}");
+
+    let child = command.spawn().wrap_err_with(|| {
+        format!("Failed to spawn process for dropping database using command: '{command_str}': ")
+    })?;
+
+    let output = child.wait_with_output().wrap_err_with(|| {
+        format!(
+            "failed waiting for spawned process to drop database using command: '{command_str}': "
+        )
+    })?;
+
+    if !output.status.success() {
+        return Err(eyre!(
+            "problem running dropdb: {}\n\n{}{}",
+            command_str,
+            String::from_utf8(output.stdout).unwrap(),
+            String::from_utf8(output.stderr).unwrap()
+        ));
+    }
+
+    Ok(true)
+}
+
 fn does_db_exist(pg_config: &PgConfig, dbname: &str) -> eyre::Result<bool> {
     let mut command = Command::new(pg_config.psql_path()?);
     command
