@@ -62,6 +62,7 @@
 #include "catalog/pg_proc.h"
 #include "catalog/pg_namespace.h"
 #include "catalog/pg_seclabel.h"
+#include "catalog/pg_statistic_ext.h"
 #include "catalog/pg_tablespace.h"
 #include "catalog/pg_trigger.h"
 #include "catalog/pg_type.h"
@@ -130,6 +131,10 @@
 #include "parser/parser.h"
 #include "parser/parsetree.h"
 #include "parser/scansup.h"
+#include "partitioning/partbounds.h"
+#include "partitioning/partdefs.h"
+#include "partitioning/partdesc.h"
+#include "partitioning/partprune.h"
 #include "plpgsql.h"
 #include "postmaster/bgworker.h"
 #include "postmaster/postmaster.h"
@@ -139,7 +144,9 @@
 #include "replication/output_plugin.h"
 #include "rewrite/rewriteHandler.h"
 #include "rewrite/rowsecurity.h"
+#include "statistics/statistics.h"
 #include "storage/block.h"
+#include "storage/buf_internals.h"
 #include "storage/bufmgr.h"
 #include "storage/buffile.h"
 #include "storage/bufpage.h"
@@ -157,6 +164,7 @@
 #include "tcop/pquery.h"
 #include "tcop/tcopprot.h"
 #include "tcop/utility.h"
+#include "tsearch/ts_cache.h"
 #include "tsearch/ts_public.h"
 #include "tsearch/ts_utils.h"
 #include "utils/acl.h" 
@@ -186,6 +194,7 @@
 #include "utils/selfuncs.h"
 #include "utils/snapmgr.h"
 #include "utils/sortsupport.h"
+#include "utils/spccache.h"
 #include "utils/catcache.h"
 #include "utils/syscache.h"
 #include "utils/tuplestore.h"
