@@ -15,11 +15,9 @@ mod tests {
 
     #[pg_test]
     fn test_anynumeric_arg() -> Result<(), pgrx::spi::Error> {
-        let numeric = Spi::get_one_with_args::<AnyNumeric>(
-            "SELECT anynumeric_arg($1);",
-            vec![(PgBuiltInOids::INT4OID.oid(), 123.into_datum())],
-        )?
-        .map(|n| n.normalize().to_string());
+        let numeric =
+            Spi::get_one_with_args::<AnyNumeric>("SELECT anynumeric_arg($1);", &[123.into()])?
+                .map(|n| n.normalize().to_string());
 
         assert_eq!(numeric, Some("123".to_string()));
 
