@@ -12,16 +12,16 @@
 mod tests {
     #[allow(unused_imports)]
     use crate as pgrx_tests;
-    use pgrx::datum::IntoDatum;
+    use pgrx::datum::DatumWithOid;
+    use pgrx::Uuid;
     use std::error::Error;
 
     use pgrx::prelude::*;
-    use pgrx::spi;
-    use pgrx::spi::Query;
+    use pgrx::spi::{self, Query};
 
     #[pg_test(error = "syntax error at or near \"THIS\"")]
     fn test_spi_failure() -> Result<(), spi::Error> {
-        Spi::connect(|client| client.select("THIS IS NOT A VALID QUERY", None, None).map(|_| ()))
+        Spi::connect(|client| client.select("THIS IS NOT A VALID QUERY", None, &[]).map(|_| ()))
     }
 
     #[pg_test]
@@ -34,7 +34,7 @@ mod tests {
     #[pg_test]
     fn test_spi_returns_primitive() -> Result<(), spi::Error> {
         let rc =
-            Spi::connect(|client| client.select("SELECT 42", None, None)?.first().get::<i32>(1))?;
+            Spi::connect(|client| client.select("SELECT 42", None, &[])?.first().get::<i32>(1))?;
 
         assert_eq!(Some(42), rc);
         Ok(())
@@ -43,7 +43,7 @@ mod tests {
     #[pg_test]
     fn test_spi_returns_str() -> Result<(), spi::Error> {
         let rc = Spi::connect(|client| {
-            client.select("SELECT 'this is a test'", None, None)?.first().get::<&str>(1)
+            client.select("SELECT 'this is a test'", None, &[])?.first().get::<&str>(1)
         })?;
 
         assert_eq!(Some("this is a test"), rc);
@@ -53,7 +53,7 @@ mod tests {
     #[pg_test]
     fn test_spi_returns_string() -> Result<(), spi::Error> {
         let rc = Spi::connect(|client| {
-            client.select("SELECT 'this is a test'", None, None)?.first().get::<&str>(1)
+            client.select("SELECT 'this is a test'", None, &[])?.first().get::<&str>(1)
         })?;
 
         assert_eq!(Some("this is a test"), rc);
@@ -63,7 +63,7 @@ mod tests {
     #[pg_test]
     fn test_spi_get_one() -> Result<(), spi::Error> {
         Spi::connect(|client| {
-            let i = client.select("SELECT 42::bigint", None, None)?.first().get_one::<i64>()?;
+            let i = client.select("SELECT 42::bigint", None, &[])?.first().get_one::<i64>()?;
             assert_eq!(Some(42), i);
             Ok(())
         })
@@ -73,7 +73,7 @@ mod tests {
     fn test_spi_get_two() -> Result<(), spi::Error> {
         Spi::connect(|client| {
             let (i, s) =
-                client.select("SELECT 42, 'test'", None, None)?.first().get_two::<i64, &str>()?;
+                client.select("SELECT 42, 'test'", None, &[])?.first().get_two::<i64, &str>()?;
 
             assert_eq!(Some(42), i);
             assert_eq!(Some("test"), s);
@@ -85,7 +85,7 @@ mod tests {
     fn test_spi_get_three() -> Result<(), spi::Error> {
         Spi::connect(|client| {
             let (i, s, b) = client
-                .select("SELECT 42, 'test', true", None, None)?
+                .select("SELECT 42, 'test', true", None, &[])?
                 .first()
                 .get_three::<i64, &str, bool>()?;
 
@@ -99,11 +99,7 @@ mod tests {
     #[pg_test]
     fn test_spi_get_two_with_failure() -> Result<(), spi::Error> {
         Spi::connect(|client| {
-            assert!(client
-                .select("SELECT 42", None, None)?
-                .first()
-                .get_two::<i64, &str>()
-                .is_err());
+            assert!(client.select("SELECT 42", None, &[])?.first().get_two::<i64, &str>().is_err());
             Ok(())
         })
     }
@@ -112,7 +108,7 @@ mod tests {
     fn test_spi_get_three_failure() -> Result<(), spi::Error> {
         Spi::connect(|client| {
             assert!(client
-                .select("SELECT 42, 'test'", None, None)?
+                .select("SELECT 42, 'test'", None, &[])?
                 .first()
                 .get_three::<i64, &str, bool>()
                 .is_err());
@@ -135,14 +131,7 @@ mod tests {
         let i = 1 as i32;
         let j = 2 as i64;
 
-        assert!(Spi::run_with_args(
-            "SELECT $1 + $2 = 3",
-            Some(vec![
-                (PgBuiltInOids::INT4OID.oid(), Some(i.into())),
-                (PgBuiltInOids::INT8OID.oid(), Some(j.into())),
-            ]),
-        )
-        .is_ok());
+        assert!(Spi::run_with_args("SELECT $1 + $2 = 3", &[i.into(), j.into(),],).is_ok());
     }
 
     #[pg_test]
@@ -157,13 +146,7 @@ mod tests {
         let i = 1 as i32;
         let j = 2 as i64;
 
-        let result = Spi::explain_with_args(
-            "SELECT $1 + $2 = 3",
-            Some(vec![
-                (PgBuiltInOids::INT4OID.oid(), Some(i.into())),
-                (PgBuiltInOids::INT8OID.oid(), Some(j.into())),
-            ]),
-        )?;
+        let result = Spi::explain_with_args("SELECT $1 + $2 = 3", &[i.into(), j.into()])?;
 
         assert!(result.0.get(0).unwrap().get("Plan").is_some());
         Ok(())
@@ -181,13 +164,13 @@ mod tests {
 
     #[pg_test]
     fn test_inserting_null() -> Result<(), pgrx::spi::Error> {
-        Spi::connect(|mut client| {
-            client.update("CREATE TABLE tests.null_test (id uuid)", None, None).map(|_| ())
+        Spi::connect_mut(|client| {
+            client.update("CREATE TABLE tests.null_test (id uuid)", None, &[]).map(|_| ())
         })?;
         assert_eq!(
             Spi::get_one_with_args::<i32>(
                 "INSERT INTO tests.null_test VALUES ($1) RETURNING 1",
-                vec![(PgBuiltInOids::UUIDOID.oid(), None)],
+                &[DatumWithOid::null::<Uuid>()],
             )?
             .unwrap(),
             1
@@ -204,15 +187,15 @@ mod tests {
 
     #[pg_test]
     fn test_cursor() -> Result<(), spi::Error> {
-        Spi::connect(|mut client| {
-            client.update("CREATE TABLE tests.cursor_table (id int)", None, None)?;
+        Spi::connect_mut(|client| {
+            client.update("CREATE TABLE tests.cursor_table (id int)", None, &[])?;
             client.update(
                 "INSERT INTO tests.cursor_table (id) \
             SELECT i FROM generate_series(1, 10) AS t(i)",
                 None,
-                None,
+                &[],
             )?;
-            let mut portal = client.open_cursor("SELECT * FROM tests.cursor_table", None);
+            let mut portal = client.open_cursor("SELECT * FROM tests.cursor_table", &[]);
 
             assert_eq!(sum_all(portal.fetch(3)?), 1 + 2 + 3);
             assert_eq!(sum_all(portal.fetch(3)?), 4 + 5 + 6);
@@ -224,16 +207,16 @@ mod tests {
 
     #[pg_test]
     fn test_cursor_prepared_statement() -> Result<(), pgrx::spi::Error> {
-        Spi::connect(|mut client| {
-            client.update("CREATE TABLE tests.cursor_table (id int)", None, None)?;
+        Spi::connect_mut(|client| {
+            client.update("CREATE TABLE tests.cursor_table (id int)", None, &[])?;
             client.update(
                 "INSERT INTO tests.cursor_table (id) \
             SELECT i FROM generate_series(1, 10) AS t(i)",
                 None,
-                None,
+                &[],
             )?;
-            let prepared = client.prepare("SELECT * FROM tests.cursor_table", None)?;
-            let mut portal = client.open_cursor(&prepared, None);
+            let prepared = client.prepare("SELECT * FROM tests.cursor_table", &[])?;
+            let mut portal = client.open_cursor(&prepared, &[]);
 
             assert_eq!(sum_all(portal.fetch(3)?), 1 + 2 + 3);
             assert_eq!(sum_all(portal.fetch(3)?), 4 + 5 + 6);
@@ -243,39 +226,34 @@ mod tests {
         })
     }
 
-    #[pg_test]
-    #[should_panic(expected = "PreparedStatementArgumentMismatch { expected: 1, got: 0 }")]
-    fn test_cursor_prepared_statement_panics_none_args() -> Result<(), pgrx::spi::Error> {
-        test_cursor_prepared_statement_panics_impl(None)
-    }
-
     #[pg_test]
     #[should_panic(expected = "PreparedStatementArgumentMismatch { expected: 1, got: 0 }")]
     fn test_cursor_prepared_statement_panics_less_args() -> Result<(), pgrx::spi::Error> {
-        test_cursor_prepared_statement_panics_impl(Some([].to_vec()))
+        test_cursor_prepared_statement_panics_impl(&[])
     }
 
     #[pg_test]
     #[should_panic(expected = "PreparedStatementArgumentMismatch { expected: 1, got: 2 }")]
     fn test_cursor_prepared_statement_panics_more_args() -> Result<(), pgrx::spi::Error> {
-        test_cursor_prepared_statement_panics_impl(Some([None, None].to_vec()))
+        test_cursor_prepared_statement_panics_impl(&[
+            DatumWithOid::null::<i32>(),
+            DatumWithOid::null::<i32>(),
+        ])
     }
 
     fn test_cursor_prepared_statement_panics_impl(
-        args: Option<Vec<Option<pg_sys::Datum>>>,
+        args: &[DatumWithOid],
     ) -> Result<(), pgrx::spi::Error> {
-        Spi::connect(|mut client| {
-            client.update("CREATE TABLE tests.cursor_table (id int)", None, None)?;
+        Spi::connect_mut(|client| {
+            client.update("CREATE TABLE tests.cursor_table (id int)", None, &[])?;
             client.update(
                 "INSERT INTO tests.cursor_table (id) \
             SELECT i FROM generate_series(1, 10) AS t(i)",
                 None,
-                None,
-            )?;
-            let prepared = client.prepare(
-                "SELECT * FROM tests.cursor_table WHERE id = $1",
-                Some([PgBuiltInOids::INT4OID.oid()].to_vec()),
+                &[],
             )?;
+            let prepared =
+                client.prepare("SELECT * FROM tests.cursor_table WHERE id = $1", &oids_of![i32])?;
             client.open_cursor(&prepared, args);
             unreachable!();
         })
@@ -283,15 +261,15 @@ mod tests {
 
     #[pg_test]
     fn test_cursor_by_name() -> Result<(), pgrx::spi::Error> {
-        let cursor_name = Spi::connect(|mut client| {
-            client.update("CREATE TABLE tests.cursor_table (id int)", None, None)?;
+        let cursor_name = Spi::connect_mut(|client| {
+            client.update("CREATE TABLE tests.cursor_table (id int)", None, &[])?;
             client.update(
                 "INSERT INTO tests.cursor_table (id) \
             SELECT i FROM generate_series(1, 10) AS t(i)",
                 None,
-                None,
+                &[],
             )?;
-            let mut cursor = client.open_cursor("SELECT * FROM tests.cursor_table", None);
+            let mut cursor = client.open_cursor("SELECT * FROM tests.cursor_table", &[]);
             assert_eq!(sum_all(cursor.fetch(3)?), 1 + 2 + 3);
             Ok::<_, spi::Error>(cursor.detach_into_name())
         })?;
@@ -315,7 +293,7 @@ mod tests {
     #[pg_test(error = "syntax error at or near \"THIS\"")]
     fn test_cursor_failure() {
         Spi::connect(|client| {
-            client.open_cursor("THIS IS NOT SQL", None);
+            client.open_cursor("THIS IS NOT SQL", &[]);
         })
     }
 
@@ -327,7 +305,7 @@ mod tests {
     #[pg_test]
     fn test_columns() -> Result<(), spi::Error> {
         Spi::connect(|client| {
-            let res = client.select("SELECT 42 AS a, 'test' AS b", None, None)?;
+            let res = client.select("SELECT 42 AS a, 'test' AS b", None, &[])?;
 
             assert_eq!(Ok(2), res.columns());
             assert_eq!(res.column_type_oid(1).unwrap(), PgOid::BuiltIn(PgBuiltInOids::INT4OID));
@@ -337,8 +315,8 @@ mod tests {
             Ok::<_, spi::Error>(())
         })?;
 
-        Spi::connect(|mut client| {
-            let res = client.update("SET TIME ZONE 'PST8PDT'", None, None)?;
+        Spi::connect_mut(|client| {
+            let res = client.update("SET TIME ZONE 'PST8PDT'", None, &[])?;
 
             assert_eq!(Err(spi::Error::NoTupleTable), res.columns());
             Ok(())
@@ -353,10 +331,9 @@ mod tests {
 
     #[pg_test]
     fn test_spi_non_mut() -> Result<(), pgrx::spi::Error> {
-        // Ensures update and cursor APIs do not need mutable reference to SpiClient
-        Spi::connect(|mut client| {
-            client.update("SELECT 1", None, None).expect("SPI failed");
-            let cursor = client.open_cursor("SELECT 1", None).detach_into_name();
+        // Ensures cursor APIs do not need mutable reference to SpiClient
+        Spi::connect(|client| {
+            let cursor = client.open_cursor("SELECT 1", &[]).detach_into_name();
             client.find_cursor(&cursor).map(|_| ())
         })
     }
@@ -366,8 +343,8 @@ mod tests {
         // Regression test to ensure a new `SpiTupTable` instance does not override the
         // effective length of an already open one due to misuse of Spi statics
         Spi::connect(|client| {
-            let a = client.select("SELECT 1", None, None)?.first();
-            let _b = client.select("SELECT 1 WHERE 'f'", None, None)?;
+            let a = client.select("SELECT 1", None, &[])?.first();
+            let _b = client.select("SELECT 1 WHERE 'f'", None, &[])?;
             assert!(!a.is_empty());
             assert_eq!(1, a.len());
             assert!(a.get_heap_tuple().is_ok());
@@ -382,8 +359,8 @@ mod tests {
         // effective length of an already open one.
         // Same as `test_open_multiple_tuptables`, but with the second tuptable being empty
         Spi::connect(|client| {
-            let a = client.select("SELECT 1 WHERE 'f'", None, None)?.first();
-            let _b = client.select("SELECT 1", None, None)?;
+            let a = client.select("SELECT 1 WHERE 'f'", None, &[])?.first();
+            let _b = client.select("SELECT 1", None, &[])?;
             assert!(a.is_empty());
             assert_eq!(0, a.len());
             assert!(a.get_heap_tuple().is_ok());
@@ -395,9 +372,8 @@ mod tests {
     #[pg_test]
     fn test_prepared_statement() -> Result<(), spi::Error> {
         let rc = Spi::connect(|client| {
-            let prepared =
-                client.prepare("SELECT $1", Some(vec![PgOid::BuiltIn(PgBuiltInOids::INT4OID)]))?;
-            client.select(&prepared, None, Some(vec![42.into_datum()]))?.first().get::<i32>(1)
+            let prepared = client.prepare("SELECT $1", &oids_of![i32])?;
+            client.select(&prepared, None, &[42.into()])?.first().get::<i32>(1)
         })?;
 
         assert_eq!(42, rc.expect("SPI failed to return proper value"));
@@ -407,9 +383,8 @@ mod tests {
     #[pg_test]
     fn test_prepared_statement_argument_mismatch() {
         let err = Spi::connect(|client| {
-            let prepared =
-                client.prepare("SELECT $1", Some(vec![PgOid::BuiltIn(PgBuiltInOids::INT4OID)]))?;
-            client.select(&prepared, None, None).map(|_| ())
+            let prepared = client.prepare("SELECT $1", &oids_of![i32])?;
+            client.select(&prepared, None, &[]).map(|_| ())
         })
         .unwrap_err();
 
@@ -422,14 +397,10 @@ mod tests {
     #[pg_test]
     fn test_owned_prepared_statement() -> Result<(), spi::Error> {
         let prepared = Spi::connect(|client| {
-            Ok::<_, spi::Error>(
-                client
-                    .prepare("SELECT $1", Some(vec![PgOid::BuiltIn(PgBuiltInOids::INT4OID)]))?
-                    .keep(),
-            )
+            Ok::<_, spi::Error>(client.prepare("SELECT $1", &oids_of![i32])?.keep())
         })?;
         let rc = Spi::connect(|client| {
-            client.select(&prepared, None, Some(vec![42.into_datum()]))?.first().get::<i32>(1)
+            client.select(&prepared, None, &[42.into()])?.first().get::<i32>(1)
         })?;
 
         assert_eq!(Some(42), rc);
@@ -444,16 +415,16 @@ mod tests {
     #[pg_test(error = "CREATE TABLE is not allowed in a non-volatile function")]
     fn test_readwrite_in_readonly() -> Result<(), spi::Error> {
         // This is supposed to run in read-only
-        Spi::connect(|client| client.select("CREATE TABLE a ()", None, None).map(|_| ()))
+        Spi::connect(|client| client.select("CREATE TABLE a ()", None, &[]).map(|_| ()))
     }
 
     #[pg_test]
     fn test_readwrite_in_select_readwrite() -> Result<(), spi::Error> {
-        Spi::connect(|mut client| {
+        Spi::connect_mut(|client| {
             // This is supposed to switch connection to read-write and run it there
-            client.update("CREATE TABLE a (id INT)", None, None)?;
+            client.update("CREATE TABLE a (id INT)", None, &[])?;
             // This is supposed to run in read-write
-            client.select("INSERT INTO a VALUES (1)", None, None)?;
+            client.select("INSERT INTO a VALUES (1)", None, &[])?;
             Ok(())
         })
     }
@@ -461,9 +432,9 @@ mod tests {
     #[pg_test(error = "CREATE TABLE is not allowed in a non-volatile function")]
     fn test_execute_prepared_statement_in_readonly() -> Result<(), spi::Error> {
         Spi::connect(|client| {
-            let stmt = client.prepare("CREATE TABLE a ()", None)?;
+            let stmt = client.prepare("CREATE TABLE a ()", &[])?;
             // This is supposed to run in read-only
-            stmt.execute(&client, Some(1), None)?;
+            stmt.execute(&client, Some(1), &[])?;
             Ok(())
         })
     }
@@ -471,19 +442,19 @@ mod tests {
     #[pg_test]
     fn test_execute_prepared_statement_in_readwrite() -> Result<(), spi::Error> {
         Spi::connect(|client| {
-            let stmt = client.prepare_mut("CREATE TABLE a ()", None)?;
+            let stmt = client.prepare_mut("CREATE TABLE a ()", &[])?;
             // This is supposed to run in read-write
-            stmt.execute(&client, Some(1), None)?;
+            stmt.execute(&client, Some(1), &[])?;
             Ok(())
         })
     }
 
     #[pg_test]
     fn test_spi_select_sees_update() -> spi::Result<()> {
-        let with_select = Spi::connect(|mut client| {
-            client.update("CREATE TABLE asd(id int)", None, None)?;
-            client.update("INSERT INTO asd(id) VALUES (1)", None, None)?;
-            client.select("SELECT COUNT(*) FROM asd", None, None)?.first().get_one::<i64>()
+        let with_select = Spi::connect_mut(|client| {
+            client.update("CREATE TABLE asd(id int)", None, &[])?;
+            client.update("INSERT INTO asd(id) VALUES (1)", None, &[])?;
+            client.select("SELECT COUNT(*) FROM asd", None, &[])?.first().get_one::<i64>()
         })?;
         let with_get_one = Spi::get_one::<i64>("SELECT COUNT(*) FROM asd")?;
 
@@ -496,7 +467,7 @@ mod tests {
         Spi::run("CREATE TABLE asd(id int)")?;
         Spi::run("INSERT INTO asd(id) VALUES (1)")?;
         let with_select = Spi::connect(|client| {
-            client.select("SELECT COUNT(*) FROM asd", None, None)?.first().get_one::<i64>()
+            client.select("SELECT COUNT(*) FROM asd", None, &[])?.first().get_one::<i64>()
         })?;
         let with_get_one = Spi::get_one::<i64>("SELECT COUNT(*) FROM asd")?;
 
@@ -506,13 +477,13 @@ mod tests {
 
     #[pg_test]
     fn test_spi_select_sees_update_in_other_session() -> spi::Result<()> {
-        Spi::connect::<spi::Result<()>, _>(|mut client| {
-            client.update("CREATE TABLE asd(id int)", None, None)?;
-            client.update("INSERT INTO asd(id) VALUES (1)", None, None)?;
+        Spi::connect_mut::<spi::Result<()>, _>(|client| {
+            client.update("CREATE TABLE asd(id int)", None, &[])?;
+            client.update("INSERT INTO asd(id) VALUES (1)", None, &[])?;
             Ok(())
         })?;
         let with_select = Spi::connect(|client| {
-            client.select("SELECT COUNT(*) FROM asd", None, None)?.first().get_one::<i64>()
+            client.select("SELECT COUNT(*) FROM asd", None, &[])?.first().get_one::<i64>()
         })?;
         let with_get_one = Spi::get_one::<i64>("SELECT COUNT(*) FROM asd")?;
 
@@ -572,7 +543,7 @@ mod tests {
     #[pg_test]
     fn can_return_borrowed_str() -> Result<(), Box<dyn Error>> {
         let res = Spi::connect(|c| {
-            let mut cursor = c.open_cursor("SELECT 'hello' FROM generate_series(1, 10000)", None);
+            let mut cursor = c.open_cursor("SELECT 'hello' FROM generate_series(1, 10000)", &[]);
             let table = cursor.fetch(10000)?;
             table.into_iter().map(|row| row.get::<&str>(1)).collect::<Result<Vec<_>, _>>()
         })?;
