@@ -20,7 +20,7 @@ use std::ptr::null_mut;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::time::Duration;
 
-pub static mut PREV_SHMEM_STARTUP_HOOK: Option<unsafe extern "C" fn()> = None;
+pub static mut PREV_SHMEM_STARTUP_HOOK: Option<unsafe extern "C-unwind" fn()> = None;
 static GOT_SIGHUP: AtomicBool = AtomicBool::new(false);
 static GOT_SIGTERM: AtomicBool = AtomicBool::new(false);
 static GOT_SIGINT: AtomicBool = AtomicBool::new(false);
@@ -52,9 +52,6 @@ bitflags! {
         const WL_POSTMASTER_DEATH  = pg_sys::WL_POSTMASTER_DEATH as i32;
         const WL_SOCKET_CONNECTED  = pg_sys::WL_SOCKET_WRITEABLE as i32;
         const WL_SOCKET_MASK       = (pg_sys::WL_SOCKET_READABLE | pg_sys::WL_SOCKET_WRITEABLE | pg_sys::WL_SOCKET_CONNECTED) as i32;
-        #[cfg(feature = "pg12")]
-        const WL_EXIT_ON_PM_DEATH  = pg_sys::WL_EXIT_ON_PM_DEATH  as i32;
-
     }
 }
 
@@ -75,7 +72,6 @@ impl BackgroundWorker {
     /// What is our name?
     pub fn get_name() -> &'static str {
         #[cfg(any(
-            feature = "pg12",
             feature = "pg13",
             feature = "pg14",
             feature = "pg15",
@@ -197,7 +193,6 @@ impl BackgroundWorker {
 
         unsafe {
             #[cfg(any(
-                feature = "pg12",
                 feature = "pg13",
                 feature = "pg14",
                 feature = "pg15",
@@ -256,23 +251,23 @@ impl BackgroundWorker {
     }
 }
 
-unsafe extern "C" fn worker_spi_sighup(_signal_args: i32) {
+unsafe extern "C-unwind" fn worker_spi_sighup(_signal_args: i32) {
     GOT_SIGHUP.store(true, Ordering::SeqCst);
     pg_sys::ProcessConfigFile(pg_sys::GucContext::PGC_SIGHUP);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
-unsafe extern "C" fn worker_spi_sigterm(_signal_args: i32) {
+unsafe extern "C-unwind" fn worker_spi_sigterm(_signal_args: i32) {
     GOT_SIGTERM.store(true, Ordering::SeqCst);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
-unsafe extern "C" fn worker_spi_sigint(_signal_args: i32) {
+unsafe extern "C-unwind" fn worker_spi_sigint(_signal_args: i32) {
     GOT_SIGINT.store(true, Ordering::SeqCst);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
-unsafe extern "C" fn worker_spi_sigchld(_signal_args: i32) {
+unsafe extern "C-unwind" fn worker_spi_sigchld(_signal_args: i32) {
     GOT_SIGCHLD.store(true, Ordering::SeqCst);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
@@ -410,7 +405,7 @@ impl TerminatingDynamicBackgroundWorker {
 /// use pgrx::bgworkers::BackgroundWorkerBuilder;
 ///
 /// #[pg_guard]
-/// pub extern "C" fn _PG_init() {
+/// pub extern "C-unwind" fn _PG_init() {
 ///     BackgroundWorkerBuilder::new("My Example BGWorker")
 ///         .set_function("background_worker_main")
 ///         .set_library("example")
@@ -419,7 +414,7 @@ impl TerminatingDynamicBackgroundWorker {
 /// }
 ///
 /// #[pg_guard]
-/// pub extern "C" fn background_worker_main(_arg: pg_sys::Datum) {
+/// pub extern "C-unwind" fn background_worker_main(_arg: pg_sys::Datum) {
 ///     // do bgworker stuff here
 /// }
 /// ```
@@ -434,7 +429,7 @@ pub struct BackgroundWorkerBuilder {
     bgw_main_arg: pg_sys::Datum,
     bgw_extra: String,
     bgw_notify_pid: pg_sys::pid_t,
-    shared_memory_startup_fn: Option<unsafe extern "C" fn()>,
+    shared_memory_startup_fn: Option<unsafe extern "C-unwind" fn()>,
 }
 
 impl BackgroundWorkerBuilder {
@@ -470,7 +465,7 @@ impl BackgroundWorkerBuilder {
     ///
     /// `startup` allows specifying shared memory initialization startup hook. Ignored
     /// if [`BackgroundWorkerBuilder::load_dynamic`] is used.
-    pub fn enable_shmem_access(mut self, startup: Option<unsafe extern "C" fn()>) -> Self {
+    pub fn enable_shmem_access(mut self, startup: Option<unsafe extern "C-unwind" fn()>) -> Self {
         self.bgw_flags = self.bgw_flags | BGWflags::BGWORKER_SHMEM_ACCESS;
         self.shared_memory_startup_fn = startup;
         self
@@ -514,7 +509,7 @@ impl BackgroundWorkerBuilder {
     /// process is started?
     ///
     /// The specified function **must** be:
-    ///     - `extern "C"`,
+    ///     - `extern "C-unwind"`,
     ///     - guarded with `#[pg_guard]`,
     ///     - take 1 argument of type `pgrx::pg_sys::Datum`, and
     ///     - return "void"
@@ -525,7 +520,7 @@ impl BackgroundWorkerBuilder {
     /// use pgrx::prelude::*;
     ///
     /// #[pg_guard]
-    /// pub extern "C" fn background_worker_main(_arg: pg_sys::Datum) {
+    /// pub extern "C-unwind" fn background_worker_main(_arg: pg_sys::Datum) {
     /// }
     /// ```
     pub fn set_function(mut self, input: &str) -> Self {
@@ -625,7 +620,6 @@ impl BackgroundWorkerBuilder {
 impl<'a> From<&'a BackgroundWorkerBuilder> for pg_sys::BackgroundWorker {
     fn from(builder: &'a BackgroundWorkerBuilder) -> Self {
         #[cfg(any(
-            feature = "pg12",
             feature = "pg13",
             feature = "pg14",
             feature = "pg15",
@@ -678,7 +672,6 @@ fn wait_latch(timeout: libc::c_long, wakeup_flags: WLflags) -> i32 {
 }
 
 #[cfg(any(
-    feature = "pg12",
     feature = "pg13",
     feature = "pg14",
     feature = "pg15",
