@@ -19,7 +19,7 @@ use std::num::NonZeroUsize;
 /// If converting a Datum to a Rust type fails, this is the set of possible reasons why.
 #[derive(thiserror::Error, Debug, Clone, PartialEq, Eq)]
 pub enum TryFromDatumError {
-    #[error("Postgres type {datum_type} ({datum_oid:?}) is not compatible with the Rust type {rust_type} ({rust_oid:?})")]
+    #[error("Postgres type {datum_type} (Oid({datum_oid})) is not compatible with the Rust type {rust_type} (Oid({rust_oid}))")]
     IncompatibleTypes {
         rust_type: &'static str,
         rust_oid: pg_sys::Oid,
@@ -209,6 +209,21 @@ impl FromDatum for pg_sys::Oid {
     }
 }
 
+impl FromDatum for pg_sys::TransactionId {
+    #[inline]
+    unsafe fn from_polymorphic_datum(
+        datum: pg_sys::Datum,
+        is_null: bool,
+        _typoid: pg_sys::Oid,
+    ) -> Option<Self> {
+        if is_null {
+            None
+        } else {
+            datum.value().try_into().ok().map(Self::from_inner)
+        }
+    }
+}
+
 /// for bool
 impl FromDatum for bool {
     #[inline]
@@ -432,7 +447,7 @@ impl FromDatum for String {
         _typoid: pg_sys::Oid,
     ) -> Option<String> {
         if is_null || datum.is_null() {
-            return None;
+            None
         } else {
             let varlena = pg_sys::pg_detoast_datum_packed(datum.cast_mut_ptr());
             let converted_varlena = convert_varlena_to_str_memoized(varlena);
