@@ -19,7 +19,7 @@ use crate::prelude::*;
 use crate::{void_mut_ptr, PgList};
 use std::ops::Deref;
 
-#[cfg(any(feature = "pg12", feature = "pg13"))]
+#[cfg(any(feature = "pg13"))]
 // JumbleState is not defined prior to postgres v14.
 // This zero-sized type is here to provide an inner type for
 // the option in post_parse_analyze_hook, but prior to v14
@@ -209,7 +209,7 @@ pub unsafe fn register_hook(hook: &'static mut (dyn PgHooks)) {
     if HOOKS.is_some() {
         panic!("PgHook instance already registered");
     }
-    #[cfg(any(feature = "pg12", feature = "pg13", feature = "pg14", feature = "pg15"))]
+    #[cfg(any(feature = "pg13", feature = "pg14", feature = "pg15"))]
     let prev_executor_check_perms_hook = pg_sys::ExecutorCheckPerms_hook
         .replace(pgrx_executor_check_perms)
         .or(Some(pgrx_standard_executor_check_perms_wrapper));
@@ -246,7 +246,7 @@ pub unsafe fn register_hook(hook: &'static mut (dyn PgHooks)) {
     });
 
     #[pg_guard]
-    unsafe extern "C" fn xact_callback(event: pg_sys::XactEvent::Type, _data: void_mut_ptr) {
+    unsafe extern "C-unwind" fn xact_callback(event: pg_sys::XactEvent::Type, _data: void_mut_ptr) {
         match event {
             pg_sys::XactEvent::XACT_EVENT_ABORT => HOOKS.as_mut().unwrap().current_hook.abort(),
             pg_sys::XactEvent::XACT_EVENT_PRE_COMMIT => {
@@ -260,7 +260,7 @@ pub unsafe fn register_hook(hook: &'static mut (dyn PgHooks)) {
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_start(query_desc: *mut pg_sys::QueryDesc, eflags: i32) {
+unsafe extern "C-unwind" fn pgrx_executor_start(query_desc: *mut pg_sys::QueryDesc, eflags: i32) {
     fn prev(query_desc: PgBox<pg_sys::QueryDesc>, eflags: i32) -> HookResult<()> {
         unsafe {
             (HOOKS.as_mut().unwrap().prev_executor_start_hook.as_ref().unwrap())(
@@ -275,7 +275,7 @@ unsafe extern "C" fn pgrx_executor_start(query_desc: *mut pg_sys::QueryDesc, efl
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_run(
+unsafe extern "C-unwind" fn pgrx_executor_run(
     query_desc: *mut pg_sys::QueryDesc,
     direction: pg_sys::ScanDirection::Type,
     count: u64,
@@ -302,7 +302,7 @@ unsafe extern "C" fn pgrx_executor_run(
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_finish(query_desc: *mut pg_sys::QueryDesc) {
+unsafe extern "C-unwind" fn pgrx_executor_finish(query_desc: *mut pg_sys::QueryDesc) {
     fn prev(query_desc: PgBox<pg_sys::QueryDesc>) -> HookResult<()> {
         unsafe {
             (HOOKS.as_mut().unwrap().prev_executor_finish_hook.as_ref().unwrap())(
@@ -316,7 +316,7 @@ unsafe extern "C" fn pgrx_executor_finish(query_desc: *mut pg_sys::QueryDesc) {
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_end(query_desc: *mut pg_sys::QueryDesc) {
+unsafe extern "C-unwind" fn pgrx_executor_end(query_desc: *mut pg_sys::QueryDesc) {
     fn prev(query_desc: PgBox<pg_sys::QueryDesc>) -> HookResult<()> {
         unsafe {
             (HOOKS.as_mut().unwrap().prev_executor_end_hook.as_ref().unwrap())(query_desc.into_pg())
@@ -327,9 +327,9 @@ unsafe extern "C" fn pgrx_executor_end(query_desc: *mut pg_sys::QueryDesc) {
     hook.executor_end(PgBox::from_pg(query_desc), prev);
 }
 
-#[cfg(any(feature = "pg12", feature = "pg13", feature = "pg14", feature = "pg15"))]
+#[cfg(any(feature = "pg13", feature = "pg14", feature = "pg15"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_check_perms(
+unsafe extern "C-unwind" fn pgrx_executor_check_perms(
     range_table: *mut pg_sys::List,
     ereport_on_violation: bool,
 ) -> bool {
@@ -351,7 +351,7 @@ unsafe extern "C" fn pgrx_executor_check_perms(
 
 #[cfg(any(feature = "pg16", feature = "pg17"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_executor_check_perms(
+unsafe extern "C-unwind" fn pgrx_executor_check_perms(
     range_table: *mut pg_sys::List,
     rte_perm_infos: *mut pg_sys::List,
     ereport_on_violation: bool,
@@ -379,9 +379,9 @@ unsafe extern "C" fn pgrx_executor_check_perms(
     .inner
 }
 
-#[cfg(any(feature = "pg12", feature = "pg13"))]
+#[cfg(feature = "pg13")]
 #[pg_guard]
-unsafe extern "C" fn pgrx_process_utility(
+unsafe extern "C-unwind" fn pgrx_process_utility(
     pstmt: *mut pg_sys::PlannedStmt,
     query_string: *const ::std::os::raw::c_char,
     context: pg_sys::ProcessUtilityContext::Type,
@@ -429,7 +429,7 @@ unsafe extern "C" fn pgrx_process_utility(
 }
 #[cfg(any(feature = "pg14", feature = "pg15", feature = "pg16", feature = "pg17"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_process_utility(
+unsafe extern "C-unwind" fn pgrx_process_utility(
     pstmt: *mut pg_sys::PlannedStmt,
     query_string: *const ::std::os::raw::c_char,
     read_only_tree: bool,
@@ -478,25 +478,8 @@ unsafe extern "C" fn pgrx_process_utility(
     .inner
 }
 
-#[cfg(feature = "pg12")]
 #[pg_guard]
-unsafe extern "C" fn pgrx_planner(
-    parse: *mut pg_sys::Query,
-    cursor_options: i32,
-    bound_params: pg_sys::ParamListInfo,
-) -> *mut pg_sys::PlannedStmt {
-    pgrx_planner_impl(parse, std::ptr::null(), cursor_options, bound_params)
-}
-
-#[cfg(any(
-    feature = "pg13",
-    feature = "pg14",
-    feature = "pg15",
-    feature = "pg16",
-    feature = "pg17"
-))]
-#[pg_guard]
-unsafe extern "C" fn pgrx_planner(
+unsafe extern "C-unwind" fn pgrx_planner(
     parse: *mut pg_sys::Query,
     query_string: *const ::std::os::raw::c_char,
     cursor_options: i32,
@@ -506,7 +489,7 @@ unsafe extern "C" fn pgrx_planner(
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_planner_impl(
+unsafe extern "C-unwind" fn pgrx_planner_impl(
     parse: *mut pg_sys::Query,
     query_string: *const ::std::os::raw::c_char,
     cursor_options: i32,
@@ -519,30 +502,12 @@ unsafe extern "C" fn pgrx_planner_impl(
         bound_params: PgBox<pg_sys::ParamListInfoData>,
     ) -> HookResult<*mut pg_sys::PlannedStmt> {
         HookResult::new(unsafe {
-            #[cfg(feature = "pg12")]
-            {
-                (HOOKS.as_mut().unwrap().prev_planner_hook.as_ref().unwrap())(
-                    parse.into_pg(),
-                    cursor_options,
-                    bound_params.into_pg(),
-                )
-            }
-
-            #[cfg(any(
-                feature = "pg13",
-                feature = "pg14",
-                feature = "pg15",
-                feature = "pg16",
-                feature = "pg17"
-            ))]
-            {
-                (HOOKS.as_mut().unwrap().prev_planner_hook.as_ref().unwrap())(
-                    parse.into_pg(),
-                    query_string,
-                    cursor_options,
-                    bound_params.into_pg(),
-                )
-            }
+            (HOOKS.as_mut().unwrap().prev_planner_hook.as_ref().unwrap())(
+                parse.into_pg(),
+                query_string,
+                cursor_options,
+                bound_params.into_pg(),
+            )
         })
     }
     let hook = &mut HOOKS.as_mut().unwrap().current_hook;
@@ -556,9 +521,9 @@ unsafe extern "C" fn pgrx_planner_impl(
     .inner
 }
 
-#[cfg(any(feature = "pg12", feature = "pg13"))]
+#[cfg(feature = "pg13")]
 #[pg_guard]
-unsafe extern "C" fn pgrx_post_parse_analyze(
+unsafe extern "C-unwind" fn pgrx_post_parse_analyze(
     parse_state: *mut pg_sys::ParseState,
     query: *mut pg_sys::Query,
 ) {
@@ -581,7 +546,7 @@ unsafe extern "C" fn pgrx_post_parse_analyze(
 
 #[cfg(any(feature = "pg14", feature = "pg15", feature = "pg16", feature = "pg17"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_post_parse_analyze(
+unsafe extern "C-unwind" fn pgrx_post_parse_analyze(
     parse_state: *mut pg_sys::ParseState,
     query: *mut pg_sys::Query,
     jumble_state: *mut JumbleState,
@@ -612,7 +577,7 @@ unsafe extern "C" fn pgrx_post_parse_analyze(
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_emit_log(error_data: *mut pg_sys::ErrorData) {
+unsafe extern "C-unwind" fn pgrx_emit_log(error_data: *mut pg_sys::ErrorData) {
     fn prev(error_data: PgBox<pg_sys::ErrorData>) -> HookResult<()> {
         HookResult::new(unsafe {
             match HOOKS.as_mut().unwrap().prev_emit_log_hook.as_ref() {
@@ -627,7 +592,7 @@ unsafe extern "C" fn pgrx_emit_log(error_data: *mut pg_sys::ErrorData) {
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_start_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_executor_start_wrapper(
     query_desc: *mut pg_sys::QueryDesc,
     eflags: i32,
 ) {
@@ -635,7 +600,7 @@ unsafe extern "C" fn pgrx_standard_executor_start_wrapper(
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_run_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_executor_run_wrapper(
     query_desc: *mut pg_sys::QueryDesc,
     direction: pg_sys::ScanDirection::Type,
     count: u64,
@@ -645,18 +610,20 @@ unsafe extern "C" fn pgrx_standard_executor_run_wrapper(
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_finish_wrapper(query_desc: *mut pg_sys::QueryDesc) {
+unsafe extern "C-unwind" fn pgrx_standard_executor_finish_wrapper(
+    query_desc: *mut pg_sys::QueryDesc,
+) {
     pg_sys::standard_ExecutorFinish(query_desc)
 }
 
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_end_wrapper(query_desc: *mut pg_sys::QueryDesc) {
+unsafe extern "C-unwind" fn pgrx_standard_executor_end_wrapper(query_desc: *mut pg_sys::QueryDesc) {
     pg_sys::standard_ExecutorEnd(query_desc)
 }
 
-#[cfg(any(feature = "pg12", feature = "pg13", feature = "pg14", feature = "pg15"))]
+#[cfg(any(feature = "pg13", feature = "pg14", feature = "pg15"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_check_perms_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_executor_check_perms_wrapper(
     _range_table: *mut pg_sys::List,
     _ereport_on_violation: bool,
 ) -> bool {
@@ -665,7 +632,7 @@ unsafe extern "C" fn pgrx_standard_executor_check_perms_wrapper(
 
 #[cfg(any(feature = "pg16", feature = "pg17"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_executor_check_perms_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_executor_check_perms_wrapper(
     _range_table: *mut pg_sys::List,
     _rte_perm_infos: *mut pg_sys::List,
     _ereport_on_violation: bool,
@@ -673,9 +640,9 @@ unsafe extern "C" fn pgrx_standard_executor_check_perms_wrapper(
     true
 }
 
-#[cfg(any(feature = "pg12", feature = "pg13"))]
+#[cfg(feature = "pg13")]
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_process_utility_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_process_utility_wrapper(
     pstmt: *mut pg_sys::PlannedStmt,
     query_string: *const ::std::os::raw::c_char,
     context: pg_sys::ProcessUtilityContext::Type,
@@ -697,7 +664,7 @@ unsafe extern "C" fn pgrx_standard_process_utility_wrapper(
 
 #[cfg(any(feature = "pg14", feature = "pg15", feature = "pg16", feature = "pg17"))]
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_process_utility_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_process_utility_wrapper(
     pstmt: *mut pg_sys::PlannedStmt,
     query_string: *const ::std::os::raw::c_char,
     read_only_tree: bool,
@@ -719,25 +686,8 @@ unsafe extern "C" fn pgrx_standard_process_utility_wrapper(
     )
 }
 
-#[cfg(feature = "pg12")]
 #[pg_guard]
-unsafe extern "C" fn pgrx_standard_planner_wrapper(
-    parse: *mut pg_sys::Query,
-    cursor_options: i32,
-    bound_params: pg_sys::ParamListInfo,
-) -> *mut pg_sys::PlannedStmt {
-    pg_sys::standard_planner(parse, cursor_options, bound_params)
-}
-
-#[cfg(any(
-    feature = "pg13",
-    feature = "pg14",
-    feature = "pg15",
-    feature = "pg16",
-    feature = "pg17"
-))]
-#[pg_guard]
-unsafe extern "C" fn pgrx_standard_planner_wrapper(
+unsafe extern "C-unwind" fn pgrx_standard_planner_wrapper(
     parse: *mut pg_sys::Query,
     query_string: *const ::std::os::raw::c_char,
     cursor_options: i32,
