@@ -557,7 +557,7 @@ impl PgMemoryContexts {
     pub fn leak_and_drop_on_delete<T>(&mut self, v: T) -> *mut T {
         use crate as pgrx;
         #[pgrx::pg_guard]
-        unsafe extern "C" fn drop_on_delete<T>(ptr: void_mut_ptr) {
+        unsafe extern "C-unwind" fn drop_on_delete<T>(ptr: void_mut_ptr) {
             let boxed = Box::from_raw(ptr as *mut T);
             drop(boxed);
         }
